'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (var i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

function create(createState) {
  var listeners = new Set();

  var setState = function setState(partialState) {
    state = Object.assign({}, state, typeof partialState === 'function' ? partialState(state) : partialState);
    listeners.forEach(function (listener) {
      return listener(state);
    });
  };

  var getState = function getState() {
    return state;
  };

  var subscribe = function subscribe(listener) {
    listeners.add(listener);
    return function () {
      listeners["delete"](listener);
    };
  };

  var destroy = function destroy() {
    listeners.clear();
    state = {};
  };

  function useStore(selector) {
    // Gets entire state if no selector was passed in
    var selectState = typeof selector === 'function' ? selector : getState; // Using functional initial b/c selected itself could be a function

    var _React$useState = React.useState(function () {
      return selectState(state);
    }),
        stateSlice = _React$useState[0],
        setStateSlice = _React$useState[1]; // Prevent useEffect from needing to run when values change by storing them in a ref object


    var refs = React.useRef({
      stateSlice: stateSlice,
      selectState: selectState
    }).current; // Update refs when needed and only after view has been updated

    React.useEffect(function () {
      refs.stateSlice = stateSlice;
      refs.selectState = selectState;
    }, [stateSlice, selectState]); // Subscribe/unsubscribe to the store only on mount/unmount

    React.useEffect(function () {
      return subscribe(function () {
        // Get fresh selected state
        var selected = refs.selectState(state);
        if (!shallowEqual(refs.stateSlice, selected)) // Refresh local slice, functional initial b/c selected itself could be a function
          setStateSlice(function () {
            return selected;
          });
      });
    }, []); // Returning the selected state slice

    return stateSlice;
  }

  var state = createState(setState, getState);
  var api = {
    destroy: destroy,
    getState: getState,
    setState: setState,
    subscribe: subscribe
  };
  var result = [useStore, api];
  return result;
}

module.exports = create;
